import unittest
import pytest
import pysubgroup as ps
from pysubgroup.tests.DataSets import get_credit_data

from pysubgroup.tests.algorithms_testing import TestAlgorithmsBase

class TestSettings:
    All = True
    Apriori = False
    SimpleDFS = False
    BestFirstSearch = False
    BeamSearch = False
    DFS_bitset = False
    DFS_set = False
    DFS_numpyset = False
    SimpleSearch = False

skip_long_running = True
class BooleanTargetBase(TestAlgorithmsBase):

    # pylint: disable=no-member
    @unittest.skipUnless(TestSettings.All or TestSettings.Apriori, 'flag not set')
    def test_Apriori(self):
        self.runAlgorithm(ps.Apriori(), "Apriori", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.SimpleDFS, 'flag not set')
    def test_SimpleDFS(self):
        self.runAlgorithm(ps.SimpleDFS(), "SimpleDFS", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.All or TestSettings.BestFirstSearch, 'flag not set')
    def test_BestFirstSearch(self):
        self.runAlgorithm(ps.BestFirstSearch(), "BestFirstSearch", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.All or TestSettings.BeamSearch, 'flag not set')
    def test_BeamSearch(self):
        self.runAlgorithm(ps.BeamSearch(beam_width=12), "BeamSearch", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.All or TestSettings.DFS_bitset, 'flag not set')
    def test_DFS_bitset(self):
        self.runAlgorithm(ps.DFS(ps.BitSetRepresentation), "DFS bitset", self.result, self.qualities, self.task)

    @pytest.mark.slow
    @unittest.skipUnless(TestSettings.SimpleSearch, 'flag not set')
    def test_SimpleSearch(self):
        self.runAlgorithm(ps.SimpleSearch(), "SimpleSearch", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.All or TestSettings.DFS_set, 'flag not set')
    def test_DFS_set(self):
        self.runAlgorithm(ps.DFS(ps.SetRepresentation), "DFS set", self.result, self.qualities, self.task)

    @unittest.skipUnless(TestSettings.All or TestSettings.DFS_numpyset, 'flag not set')
    def test_DFS_numpy_sets(self):
        self.runAlgorithm(ps.DFS(ps.NumpySetRepresentation), "DFS numpyset", self.result, self.qualities, self.task)

    def evaluate_result(self, algorithm_result, result, qualities):
        df = algorithm_result.to_dataframe()
        self.assertTrue(all(algorithm_result.task.constraints[0].min_support <= df['size_sg']))
        TestAlgorithmsBase.evaluate_result(self, algorithm_result, result, qualities)
    # pylint: enable=no-member
class TestAlgorithms(BooleanTargetBase, unittest.TestCase):
    def setUp(self):
        NS_checking = ps.EqualitySelector("checking_status", b"<0")
        NS_foreign_worker = ps.EqualitySelector("foreign_worker", b"yes")
        NS_other_parties = ps.EqualitySelector("other_parties", b"none")
        NS_savings_status = ps.EqualitySelector("savings_status", b"<100")
        NS_payment_plans = ps.EqualitySelector("other_payment_plans", b"none")
        self.result = [ps.Conjunction([NS_checking, NS_foreign_worker]),
                       ps.Conjunction([NS_checking]),
                       ps.Conjunction([NS_checking, NS_other_parties, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_other_parties]),
                       ps.Conjunction([NS_checking, NS_savings_status, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_savings_status]),
                       ps.Conjunction([NS_checking, NS_foreign_worker, NS_payment_plans]),
                       ps.Conjunction([NS_checking, NS_payment_plans]),
                       ps.Conjunction([NS_foreign_worker, NS_savings_status]),
                       ps.Conjunction([NS_foreign_worker, NS_other_parties, NS_savings_status]),
                       ]
        self.qualities = [0.055299999999999995,
                          0.05280000000000001,
                          0.052300000000000006,
                          0.05059999999999999,
                          0.04959999999999999,
                          0.048299999999999996,
                          0.0426,
                          0.04,
                          0.03869999999999999,
                          0.03750000000000001]
        data = get_credit_data()
        target = ps.BinaryTarget('class', b'bad')
        searchSpace = ps.create_nominal_selectors(data, ignore=['class'])
        self.task = ps.SubgroupDiscoveryTask(data, target, searchSpace, result_set_size=10, depth=5, qf=ps.StandardQF(1.0), constraints=[ps.MinSupportConstraint(200)])

#   0.055299999999999995:        checking_status=='b'<0'' AND foreign_worker=='b'yes''
#   0.05280000000000001: checking_status=='b'<0''
#   0.052300000000000006:        checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_parties=='b'none''
#   0.05059999999999999: checking_status=='b'<0'' AND other_parties=='b'none''
#   0.04959999999999999: checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND savings_status=='b'<100''
#   0.048299999999999996:        checking_status=='b'<0'' AND savings_status=='b'<100''
#   0.0426:      checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_payment_plans=='b'none''
#   0.04:        checking_status=='b'<0'' AND other_payment_plans=='b'none''
#   0.03869999999999999: foreign_worker=='b'yes'' AND savings_status=='b'<100''
#   0.03750000000000001: foreign_worker=='b'yes'' AND other_parties=='b'none'' AND savings_status=='b'<100''


# also includes numeric attributes and has 12 targets



if __name__ == '__main__':

    #unittest.main()
    suites = []
    suites.append(unittest.TestLoader().loadTestsFromTestCase(TestAlgorithms))
    #suites.append(unittest.TestLoader().loadTestsFromTestCase(TestAlgorithms2))
    #suites.append( unittest.TestLoader().loadTestsFromTestCase(TestAlgorithms3))
    complete_suite = unittest.TestSuite(suites)
    unittest.TextTestRunner(verbosity=2).run(complete_suite)

    #import cProfile
    # p=cProfile.Profile()
    # p.enable()
    # t=TestAlgorithms()
    # t.setUp()
    # t.test_BSD()

    # p.disable()
    # p.dump_stats(r"E:\SGD\profile2.prof")

# 0.055299999999999995:   checking_status=b'<0' AND foreign_worker=b'yes'
# 0.05280000000000001:    checking_status=b'<0'
# 0.052300000000000006:   checking_status=b'<0' AND other_parties=b'none' AND foreign_worker=b'yes'
# 0.05059999999999999:    checking_status=b'<0' AND other_parties=b'none'
# 0.04959999999999999:    checking_status=b'<0' AND savings_status=b'<100' AND foreign_worker=b'yes'
# 0.048299999999999996:   checking_status=b'<0' AND savings_status=b'<100'
# 0.04660000000000001:    checking_status=b'<0' AND savings_status=b'<100' AND other_parties=b'none' AND foreign_worker=b'
# 0.04550000000000001:    checking_status=b'<0' AND job=b'skilled' AND foreign_worker=b'yes'
# 0.0452:                 checking_status=b'<0' AND savings_status=b'<100' AND other_parties=b'none'
# 0.044399999999999995:   checking_status=b'<0' AND job=b'skilled'

# <<< best 12 for a=1
# 0.055299999999999995: checking_status=='b'<0'' AND foreign_worker=='b'yes''
# 0.05280000000000001:  checking_status=='b'<0''
# 0.052300000000000006: checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_parties=='b'none''
# 0.05059999999999999:  checking_status=='b'<0'' AND other_parties=='b'none''
# 0.04959999999999999:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND savings_status=='b'<100''
# 0.04870000000000001:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND num_dependents==1.0
# 0.048299999999999996: checking_status=='b'<0'' AND savings_status=='b'<100''
# 0.0474:               checking_status=='b'<0'' AND num_dependents==1.0
# 0.04660000000000001:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_parties=='b'none'' AND savings_status=='b'<100''
# 0.04550000000000001:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled''


# <<< best 12 a=0.5
# 0.11457431093955019:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled'' AND other_parties=='b'none'' AND savings_status=='b'<100''
# 0.113713540226172:    checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled'' AND savings_status=='b'<100''
# 0.11201325679119281:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled''
# 0.1117538749727658:   checking_status=='b'<0'' AND job=='b'skilled'' AND other_parties=='b'none'' AND savings_status=='b'<100''
# 0.11161046793076415:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled'' AND other_parties=='b'none''
# 0.11145710640046322:  checking_status=='b'<0'' AND job=='b'skilled'' AND savings_status=='b'<100''
# 0.11045259291161472:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_parties=='b'none'' AND savings_status=='b'<100''
# 0.10929088624672183:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND other_parties=='b'none''
# 0.10875519439407161:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND savings_status=='b'<100''
# 0.10866138825404954:  checking_status=='b'<0'' AND foreign_worker=='b'yes''
# 0.10832735026213287:  checking_status=='b'<0'' AND foreign_worker=='b'yes'' AND job=='b'skilled'' AND num_dependents==1.0 AND savings_status=='b'<100''
# 0.10813405094128754:  checking_status=='b'<0'' AND job=='b'skilled'' AND other_parties=='b'none''



import unittest
import pysubgroup as ps
from pysubgroup.tests.DataSets import get_credit_data
from pysubgroup.tests.algorithms_testing import TestAlgorithmsBase

skip_long_running = True

class BooleanTargetBase(TestAlgorithmsBase):
    # pylint: disable=no-member
    @unittest.skipIf(skip_long_running, "as skip_long_running flag is True")
    def test_GeneralisingBFS(self):
        self.runAlgorithm(ps.GeneralisingBFS(), "GeneralisingBFS", self.result, self.qualities, self.task)

    def test_GeneralisingApriori(self):
        algorithm = ps.Apriori(combination_name='Disjunction')
        algorithm.optimistic_estimate_name = 'optimistic_generalisation'
        self.runAlgorithm(algorithm, "GeneralisingApriori", self.result, self.qualities, self.task)
    # pylint: enable=no-member

class TestAlgorithms(BooleanTargetBase, unittest.TestCase):
    def setUp(self):
        NS_checking = ps.EqualitySelector("checking_status", b"<0")
        NS_checking2 = ps.EqualitySelector("checking_status", b"0<=X<200")
        NS_other_parties = ps.EqualitySelector("other_parties", b"co applicant")
        NS_other = ps.EqualitySelector("purpose", b'other')
        NS_repairs = ps.EqualitySelector("purpose", b'repairs')
        NS_purpose = ps.EqualitySelector("purpose", b'business')

        NS_history = ps.EqualitySelector("credit_history", b"no credits/all paid")
        NS_history2 = ps.EqualitySelector("credit_history", b"all paid")
        NS_empl = ps.EqualitySelector("employment", b"unemployed")
        NS_job = ps.EqualitySelector("job", b"unemp/unskilled non res")
        NS_bank = ps.EqualitySelector("other_payment_plans", b"bank")
        self.result = [ps.Disjunction([NS_checking, NS_checking2, NS_bank]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_history]),
                       ps.Disjunction([NS_checking, NS_checking2]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_other]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_repairs]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_empl]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_other_parties]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_history2]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_purpose]),
                       ps.Disjunction([NS_checking, NS_checking2, NS_job]),
                       ]
        self.qualities = [0.0779,
                          0.07740000000000002,
                          0.0771,
                          0.07680000000000001,
                          0.07670000000000002,
                          0.0767,
                          0.07660000000000003,
                          0.07650000000000003,
                          0.07650000000000001,
                          0.07600000000000001]
        data = get_credit_data()
        target = ps.BinaryTarget('class', b'bad')
        searchSpace = ps.create_nominal_selectors(data, ignore=['class'])
        self.task = ps.SubgroupDiscoveryTask(data, target, searchSpace, result_set_size=10, depth=3, qf=ps.StandardQF(1.0))


if __name__ == '__main__':
    suite1 = unittest.TestLoader().loadTestsFromTestCase(TestAlgorithms)
    complete_suite = unittest.TestSuite([suite1])
    unittest.TextTestRunner(verbosity=2).run(complete_suite)
#depth = 5, a=1.0
#   0.07800000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_parties=='b'co applicant'' OR other_payment_plans=='b'bank''
#   0.07800000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_parties=='b'co applicant'' OR other_payment_plans=='b'bank'' OR purpose=='b'other''
#   0.07790000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'no credits/all paid'' OR other_parties=='b'co applicant'' OR other_payment_plans=='b'bank''
#   0.0779:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_payment_plans=='b'bank''
#   0.0779:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_payment_plans=='b'bank'' OR purpose=='b'other''
#   0.07780000000000002: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'no credits/all paid'' OR other_payment_plans=='b'bank''
#   0.07780000000000002: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'no credits/all paid'' OR other_payment_plans=='b'bank'' OR purpose=='b'other''
#   0.07770000000000002: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'no credits/all paid'' OR other_payment_plans=='b'bank'' OR purpose=='b'repairs''
#   0.0776:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_parties=='b'co applicant'' OR other_payment_plans=='b'bank'' OR purpose=='b'repairs''
#   0.07750000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_payment_plans=='b'bank'' OR purpose=='b'repairs''
#   0.07750000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'all paid'' OR other_parties=='b'co applicant'' OR other_payment_plans=='b'bank''

#depth = 3, a=1.0
#  0.0779:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_payment_plans=='b'bank''
#  0.07740000000000002: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'no credits/all paid''
#  0.0771:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0''
#  0.07680000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR purpose=='b'other''
#  0.07670000000000002: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR purpose=='b'repairs''
#  0.0767:              checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR employment=='b'unemployed''
#  0.07660000000000003: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR other_parties=='b'co applicant''
#  0.07650000000000003: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR credit_history=='b'all paid''
#  0.07650000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR purpose=='b'business''
#  0.07600000000000001: checking_status=='b'0<=X<200'' OR checking_status=='b'<0'' OR job=='b'unemp/unskilled non res''


from timeit import default_timer as timer
import pysubgroup as ps
from pysubgroup.tests.DataSets import get_credit_data

if __name__ == '__main__':
    data = get_credit_data()

    target = ps.BinaryTarget('class', b'bad')
    searchSpace = ps.create_nominal_selectors(data, ignore=['class'])
    task = ps.SubgroupDiscoveryTask(data, target, searchSpace, result_set_size=10, depth=4, qf=ps.ChiSquaredQF())

    print("running DFS")
    start = timer()
    result = ps.SimpleDFS().execute(task)
    end = timer()
    print("Time elapsed: ", (end - start))
    for (q, sg) in result:
        print(str(q) + ":\t" + str(sg.subgroup_description))



from setuptools import setup

setup(
    name='pysubgroup',
    version='0.7.3',
    packages=['pysubgroup', 'pysubgroup.tests'],
    package_data={'pysubgroup': ['data/credit-g.arff', 'data/titanic.csv']},
    url='http://florian.lemmerich.net/pysubgroup',
    license='',
    author='Florian Lemmerich, Felix Stamm, Martin Becker',
    author_email='florian@lemmerich.net',
    description='pysubgroup is a Python library for the data analysis task of subgroup discovery.',
    install_requires=[
        'pandas>=0.24.0',
        'scipy',
        'numpy',
        'matplotlib'
    ],
    tests_require=['pytest'],
    python_requires='>=3.6',
    classifiers=['Intended Audience :: Science/Research',
                 'Intended Audience :: Developers',
                 'Programming Language :: Python',
                 'Topic :: Software Development',
                 'Topic :: Scientific/Engineering',
                 'Operating System :: Microsoft :: Windows',
                 'Operating System :: POSIX',
                 'Operating System :: Unix',
                 'Operating System :: MacOS',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 ('Programming Language :: Python :: '
                  'Implementation :: CPython')
                 ],
)



import unittest
from collections import namedtuple
import numpy as np
import pandas as pd
import pysubgroup as ps
from pysubgroup.tests.DataSets import get_credit_data
from pysubgroup.tests.algorithms_testing import TestAlgorithmsBase

class TestGeneralisationAwareQf(unittest.TestCase):
    def setUp(self):
        self.qf = ps.CountQF()
        self.ga_qf = ps.GeneralizationAwareQF(self.qf)
        self.prepare_df()
    

    def prepare_df(self):
        A = np.array([0, 0, 1, 1, 0, 0, 1, 1, 1, 1], dtype=bool)
        self.A1 = ps.EqualitySelector("columnA", True)
        self.A0 = ps.EqualitySelector("columnA", False)

        B = np.array(["A", "B", "C", "C", "B", "A", "D", "A", "A", "A"])
        self.BA = ps.EqualitySelector("columnB", "A")
        self.BC = ps.EqualitySelector("columnB", "C")
        self.df = pd.DataFrame.from_dict({'columnA': A, 'columnB':B, 'columnC': np.array([[0, 1] for _ in range(5)]).flatten()})


    def test_CountTarget1(self):
        df = self.df
        target = ps.FITarget()
        self.ga_qf.calculate_constant_statistics(df, target)

        ga_score = self.ga_qf.evaluate(ps.Conjunction([self.A1]), target, df)

        A1_score = self.qf.evaluate(ps.Conjunction([self.A1]), target, df)
        zero_score = self.qf.evaluate(ps.Conjunction([]), target, df)

        self.assertEqual(ga_score, A1_score-zero_score)

        ga2_score = self.ga_qf.evaluate(ps.Conjunction([self.A1]), target, df)

        self.assertEqual(ga2_score, ga_score)


    def test_CountTarget2(self):
        df = self.df
        target = ps.FITarget()
        self.ga_qf.calculate_constant_statistics(df, None)

        ga_score = self.ga_qf.evaluate(ps.Conjunction([self.A1, self.BA]), target,  df)

        A_B_score = self.qf.evaluate(ps.Conjunction([self.A1, self.BA]), target, df)
        zero_score = self.qf.evaluate(ps.Conjunction([]), target, df)

        self.assertEqual(ga_score, A_B_score-zero_score)



class TestGeneralisationAware_StandardQf(unittest.TestCase):
    def setUp(self):
        self.df = None
        self.A1 = None
        self.BA = None
        TestGeneralisationAwareQf.prepare_df(self)
        self.ga_qf = ps.GeneralizationAware_StandardQF(0)

    def test_simple(self):
        target = ps.BinaryTarget('columnC', 1)
        qf = ps.StandardQF(0)
        qf.calculate_constant_statistics(self.df, target)

        self.ga_qf.calculate_constant_statistics(self.df, target)

        #print(qf.calculate_statistics(self.A1, self.df))
        #print(qf.calculate_statistics(self.BA, self.df))
        #print(qf.calculate_statistics(ps.Conjunction([self.A1, self.BA]), self.df))
        #print(qf.calculate_statistics(slice(None), self.df))
        ga_stat = self.ga_qf.calculate_statistics(ps.Conjunction([self.A1, self.BA]), target, self.df)

        self.assertEqual(ga_stat.subgroup_stats, ps.SimplePositivesQF.tpl(3, 2))
        self.assertEqual(ga_stat.generalisation_stats, ps.SimplePositivesQF.tpl(5, 3))
        # Ensure cache works properly
        self.assertEqual(ga_stat, self.ga_qf.calculate_statistics(ps.Conjunction([self.A1, self.BA]), target, self.df))

        ga_score = self.ga_qf.evaluate(ps.Conjunction([self.A1, self.BA]), target, self.df)
        ga_score2 = self.ga_qf.evaluate(ps.Conjunction([self.A1, self.BA]), target, self.df)

        self.assertEqual(ga_score, ga_score2)
        self.assertAlmostEqual(ga_score, 0.06666666666666)


class TestAlgorithms(TestAlgorithmsBase, unittest.TestCase):
    # TODO properly specify desired result
    def setUp(self):
        NS_checking = ps.EqualitySelector("checking_status", b"<0")
        NS_foreign_worker = ps.EqualitySelector("foreign_worker", b"yes")
        NS_other_parties = ps.EqualitySelector("other_parties", b"none")
        NS_savings_status = ps.EqualitySelector("savings_status", b"<100")
        NS_job = ps.EqualitySelector("job", b"skilled")
        self.result = [ps.Conjunction([NS_checking, NS_foreign_worker]),
                       ps.Conjunction([NS_checking]),
                       ps.Conjunction([NS_checking, NS_other_parties, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_other_parties]),
                       ps.Conjunction([NS_checking, NS_savings_status, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_savings_status]),
                       ps.Conjunction([NS_checking, NS_savings_status, NS_other_parties, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_job, NS_foreign_worker]),
                       ps.Conjunction([NS_checking, NS_savings_status, NS_other_parties]),
                       ps.Conjunction([NS_checking, NS_job]),
                       ]
        self.qualities = [0.055299999999999995,
                          0.05280000000000001,
                          0.052300000000000006,
                          0.05059999999999999,
                          0.04959999999999999,
                          0.048299999999999996,
                          0.04660000000000001,
                          0.04550000000000001,
                          0.0452,
                          0.044399999999999995]
        data = get_credit_data()
        target = ps.BinaryTarget('class', b'bad')
        searchSpace = ps.create_nominal_selectors(data, ignore=['class'])
        self.task = ps.SubgroupDiscoveryTask(data, target, searchSpace, result_set_size=10, depth=3, qf=ps.GeneralizationAwareQF(ps.StandardQF(1.0)))

    @unittest.skip
    def test_GA_SimpleDFS(self):
        self.runAlgorithm(ps.SimpleDFS(), "SimpleDFS", self.result, self.qualities, self.task)

    @unittest.skip
    def test_StandardQF_GA_SimpleDFS(self):
        self.task.qf = ps.GeneralizationAware_StandardQF(0.5)
        self.runAlgorithm(ps.SimpleDFS(), "Standard_SimpleDFS", self.result, self.qualities, self.task)
        print(self.task.qf.cache)


if __name__ == '__main__':
    unittest.main()



from functools import partial

import pandas as pd
import numpy as np
from scipy.cluster.hierarchy import dendrogram, linkage
from scipy.spatial.distance import squareform
from matplotlib import pyplot as plt

import pysubgroup as ps


def plot_sgbars(result_df, _, ylabel="target share", title="Discovered Subgroups", dynamic_widths=False, _suffix=""):
    shares_sg = result_df["target_share_sg"]
    shares_compl = result_df["target_share_complement"]
    sg_relative_sizes = result_df["relative_size_sg"]
    x = np.arange(len(result_df))

    base_width = 0.8
    if dynamic_widths:
        width_sg = 0.02 + base_width * sg_relative_sizes
        width_compl = base_width - width_sg
    else:
        width_sg = base_width / 2
        width_compl = base_width / 2

    fig, ax = plt.subplots()
    rects1 = ax.bar(x, shares_sg, width_sg, align='edge')
    rects2 = ax.bar(x + width_sg, shares_compl, width_compl, align='edge', color='#61b76f')

    ax.set_ylabel(ylabel)
    ax.set_title(title)
    ax.set_xticks(x + base_width / 2)
    ax.set_xticklabels(result_df.index, rotation=90)

    ax.legend((rects1[0], rects2[0]), ('subgroup', 'complement'))
    fig.set_size_inches(12, len(result_df))

    return fig


def plot_roc(result_df, data, qf=ps.StandardQF(0.5), levels=40, annotate=False):
    instances_dataset = len(data)
    positives_dataset = np.max(result_df['positives_dataset'])
    negatives_dataset = instances_dataset - positives_dataset

    xlist = np.linspace(0.01, 0.99, 100)
    ylist = np.linspace(0.01, 0.99, 100)
    X, Y = np.meshgrid(xlist, ylist)
    f = np.vectorize(partial(qf.evaluate, instances_dataset, positives_dataset), otypes=[np.float])
    Z = f(X * negatives_dataset + Y * positives_dataset, Y * positives_dataset)
    max_val = np.max([np.max(Z), -np.min(Z)])

    fig, ax = plt.subplots()
    cm = plt.cm.get_cmap("bwr")

    plt.contourf(X, Y, Z, levels, cmap=cm, vmin=-max_val, vmax=max_val)

    for i, sg in result_df.iterrows():
        rel_positives_sg = sg['positives_sg'] / positives_dataset
        rel_negatives_sg = (sg['size_sg'] - sg['positives_sg']) / negatives_dataset
        ax.plot(rel_negatives_sg, rel_positives_sg, 'ro', color='black')
        if annotate:
            label_margin = 0.01
            ax.annotate(str(i), (rel_negatives_sg + label_margin, rel_positives_sg + label_margin))

    # plt.colorbar(cp)
    plt.title('Discovered subgroups')
    plt.xlabel('False Positive Rate')
    plt.ylabel('True Positive Rate')

    return fig


def plot_npspace(result_df, data, annotate=True, fixed_limits=False):

    fig, ax = plt.subplots()

    for i, sg in result_df.iterrows():
        target_share_sg = sg['target_share_sg']
        size_sg = sg['size_sg']
        ax.plot(size_sg, target_share_sg, 'ro', color='black')
        if annotate:
            ax.annotate(str(i), (size_sg + 5, target_share_sg + 0.001))

    if fixed_limits:
        plt.xlim((0, len(data)))
        plt.ylim((0, 1))

    plt.title('Discovered subgroups')
    plt.xlabel('Size of Subgroup')
    plt.ylabel('Target Share Subgroup')

    return fig


def plot_distribution_numeric(sg, data, bins):
    fig, _ = plt.subplots()
    target_values_sg = data[sg.covers(data)][sg.target.get_attributes()].values
    target_values_data = data[sg.target.get_attributes()].values
    plt.hist(target_values_sg, bins, alpha=0.5, label=str(sg.subgroup_description), density=True)
    plt.hist(target_values_data, bins, alpha=0.5, label="Overall Data", density=True)
    plt.legend(loc='upper right')
    return fig


def compare_distributions_numeric(sgs, data, bins):
    fig, _ = plt.subplots()
    for sg in sgs:
        target_values_sg = data[sg.covers(data)][sg.target.get_attributes()].values
        plt.hist(target_values_sg, bins, alpha=0.3, label=str(sg.subgroup_description), density=True)
    plt.legend(loc='upper right')
    return fig


def similarity_sgs(sgd_results, data, color=True):
    sgs = [x[1] for x in sgd_results]
    #sgNames = [str(sg.subgroup_description) for sg in sgs]
    dists = [[ps.overlap(sg, sg2, data) for sg2 in sgs] for sg in sgs]
    dist_df = pd.DataFrame(dists)
    if color:
        dist_df = dist_df.style.background_gradient()
    return dist_df


def similarity_dendrogram(result, data):
    fig, _ = plt.subplots()
    dist_df = similarity_sgs(result, data, color=False)
    mat = 1 - dist_df.values
    dists = squareform(mat)
    linkage_matrix = linkage(dists, "single")
    dendrogram(linkage_matrix, labels=dist_df.index)
    return fig

def supportSetVisualization(result, in_order=True, drop_empty=True):
    df = result.task.data
    n_items = len(result.task.data)
    n_SGDs = len(result.results)
    covs = np.zeros((n_items, n_SGDs), dtype=bool)
    for i, (_, r, _) in enumerate(result.to_subgroups):
        covs[:, i] = r.covers(df)

    img_arr = covs.copy()

    sort_inds_x = np.argsort(np.sum(covs, axis=1))[::-1]
    img_arr = img_arr[sort_inds_x, :]
    if not in_order:
        sort_inds_y = np.argsort(np.sum(covs, axis=0))
        img_arr = img_arr[:, sort_inds_y]
    if drop_empty:
        keep_entities = np.sum(img_arr, axis=1) > 0
        print("Discarding {} entities that are not covered".format(n_items - np.count_nonzero(keep_entities)))
        img_arr = img_arr[keep_entities, :]
    return img_arr.T



from timeit import default_timer as timer
import abc
import pysubgroup as ps

class TestAlgorithmsBase(abc.ABC):

    # pylint: disable=no-member
    def evaluate_result(self, algorithm_result, result, qualities):
        self.assertTrue(isinstance(algorithm_result, ps.SubgroupDiscoveryResult))
        algorithm_result.to_dataframe()
        algorithm_result = algorithm_result.to_descriptions()
        for (q, sg) in algorithm_result:
            print("   " + str(q) + ":\t" + str(sg))
        # compare length such that zip works correctly
        self.assertEqual(len(algorithm_result), len(result))
        self.assertEqual(len(algorithm_result), len(qualities))

        for (algorithm_q, algorithm_SG), expected_q, expected_SGD in zip(algorithm_result, qualities, result):
            self.assertEqual(repr(algorithm_SG), repr(expected_SGD))
            self.assertEqual(algorithm_q, expected_q)


    def runAlgorithm(self, algorithm, name, result, qualities, task):
        print()
        print("Running " + name)
        start = timer()
        algorithm_result = algorithm.execute(task)
        end = timer()
        print("   Runtime for {}: {}".format(name, end - start))

        if hasattr(self.task.qf, 'calls'):
            print('   Number of call to qf:', self.task.qf.calls)
        print()
        self.evaluate_result(algorithm_result, result, qualities)
        return algorithm_result
    # pylint: enable=no-member



import pkg_resources
from io import StringIO
import pandas as pd
from scipy.io import arff


def get_credit_data():
    s_io = StringIO(str(pkg_resources.resource_string('pysubgroup', 'data/credit-g.arff'), 'utf-8'))
    return pd.DataFrame(arff.loadarff(s_io)[0])


def get_titanic_data():
    s_io = StringIO(str(pkg_resources.resource_string('pysubgroup', 'data/titanic.csv'), 'utf-8'))
    return pd.read_csv(s_io, sep="\t", header=[0])
